function [clat,slat,clon,slon] = dipole(time)
% DIPOLE Calculates the Earth's dipole lat/lng at the specified time
%   This function is intended for internal use by the GEO2MAG and MAG2GEO
%   functions.
%
%	Converting to geomagnetic coordinates (MAG) requires knowledge
%	of the Earth's dipole vector (in MAG coordinates, the Z-axis is
%	parallel to the Earth's dipole).
%
%	In theory, we want the latitude and longitude of the dipole 
%	axis; in reality, however, it is much quicker and easier to
%	calculate the sine and cosine of these values.
%
%------------------------------------------------------------------------------%
%
%   Version 1.0  Date: 2016/02/05
%
%   Copyright 2016 Aaron Hendry <aaron@hendry.co.nz>
%   Otago University Space Physics Department, Dunedin, New Zealand
%       <http://www.physics.otago.ac.nz/space>
%   Released under the Simplified BSD License:
%       <http://opensource.org/licenses/BSD-2-Clause>
%
%------------------------------------------------------------------------------%

persistent clatp slatp clonp slonp curr_time

% We have to call this fairly frequently, and we can significantly
% cut down on the calling time by using persistant variables 
if ~isempty(curr_time) && curr_time == time
    clat = clatp;
    slat = slatp;
    clon = clonp;
    slon = slonp;
	return
end

[g,h] = igrf_gh(time);
curr_time = time;

sq = g(2)*g(2) + h(2)*h(2);
sqq = sqrt(sq);
sqr = sqrt(sq + g(1)*g(1));

clatp = -g(1)/sqr;
slatp = sqq/sqr;
clonp = -g(2)/sqq;
slonp = -h(2)/sqq;

clat = clatp;
slat = slatp;
clon = clonp;
slon = slonp;