function [lat_cgm,lon_cgm,lval] = geo2cgm(lat,lon,height,time)
%GEO2CGM Converts geocentric units to corrected geomagnetic (CGM) units
%   [LAT_CGM,LON_CGM,LVAL] = GEO2CGM(LAT_GEO,LON_GEO,HEIGHT,TIME) converts the
%   passed spherical geographic coordinates to corrected geomagnetic (CGM)
%   coordinates. LAT_GEO and LON_GEO give the geographic latitude and longitudes
%   to calculate the CGM coordinates at in degrees. HEIGHT gives the altitude
%   above the Earth's surface (in km) at which to calculate the CGM units. TIME
%   is a MATLAB DATENUM variable that gives the epoch on which to calculate the
%   CGM units. LAT_GEO and LON_GEO can be scalars, vectors, or matrices, but
%   must either be the same size or scalars. HEIGHT and TIME must be scalar
%   values.
%
%   The GEO-CGM code on which this function is based mentions the original code
%   as being described in:
%
%  Software for computations of geomagnetic field and related coordinate systems
%  Tsyganenko et al, Soviet Geophysical Committee, Special Report, 58 pp.,
%  Moscow, 1987.
%
%   though I have not been able to find a copy of this document. The same
%   algorithm is described in:
%
%   Gustafsson, G., N. Papitashvili, and V. Papitashvili (1992), A revised
%   corrected geomagnetic coordinate system for Epochs 1985 and 1990, in:
%   Journal of Atmospheric and Terrestrial Physics 54(1112), 16091631,
%   doi:http://dx.doi.org/10.1016/0021-9169(92)90167-J.
%
%------------------------------------------------------------------------------%
%
%   Version 1.0  Date: 2016/02/05
%
%   Copyright 2016 Aaron Hendry <aaron@hendry.co.nz>
%   Otago University Space Physics Department, Dunedin, New Zealand
%       <http://www.physics.otago.ac.nz/space>
%   Released under the Simplified BSD License:
%       <http://opensource.org/licenses/BSD-2-Clause>
%
%------------------------------------------------------------------------------%

re = 6371.2;

R = ones(size(lat)).*height;
R = (R+re)./re;

if numel(lat)~=length(lat)
    if numel(lat) > 300000 % Arbitrary cutoff
        lval = nan(size(lat));
        lat_cgm = nan(size(lat));
        lon_cgm = nan(size(lat));
        for i=1:size(lat,1);
            [lval(i,:),lat_cgm(i,:),lon_cgm(i,:)] = trace_fieldline(R(i,:),lat(i,:),lon(i,:),time);
        end
    else
        sz = size(lat);
        [lval,lat_cgm,lon_cgm] = trace_fieldline(R(:),lat(:),lon(:),time);
        lval = reshape(lval,sz);
        lat_cgm = reshape(lat_cgm,sz);
        lon_cgm = reshape(lon_cgm,sz);
    end
else
    [lval,lat_cgm,lon_cgm] = trace_fieldline(R,lat,lon,time);
end

if ~any(isnan(lat_cgm(:)))
    return
end

% nlat = lat([zeros(size(lat,1),1),df]==-1);
% nlatc = lat_cgm([zeros(size(lat,1),1),df]==-1);
% nlon = lon([zeros(size(lat,1),1),df]==-1);
% slat = lat(df==1);
% slatc = lat_cgm(df==1);
% slon = lon(df==1);
% 
% [nlatdip,nlondip] = dip_equator(nlat,nlon,R(1),nlatc,time);
% [slatdip,slondip] = dip_equator(slat,slon,R(1),slatc,time);
% lonrange = unique(lon);
% nlatdip_int = interp1(rad2deg(nlondip),rad2deg(nlatdip),lonrange);
% slatdip_int = interp1(rad2deg(slondip),rad2deg(slatdip),lonrange);
% latdip = (nlatdip_int+slatdip_int)/2;
% londip = lonrange;

filt = isnan(lat_cgm);
latf = lat(filt);
if numel(lon)==1
    filt = 1;
end
[ulon,~,ind] = unique(lon(filt));
slatc = zeros(size(ulon));
slonc = zeros(size(ulon));
slvalc = zeros(size(ulon));
nlatc = zeros(size(ulon));
nlonc = zeros(size(ulon));
nlvalc = zeros(size(ulon));
slat = zeros(size(ulon));
nlat = zeros(size(ulon));

for i=1:numel(ulon)
    lat_range = -30:30;
    lon_range = ones(size(lat_range)).*ulon(i);
    R = ones(size(lat_range)).*R(1);
    [temp_lval,temp_lat,temp_lon] = trace_fieldline(R,lat_range,lon_range,time);
    while isnan(temp_lat(1))
        lat_range(1) = lat_range(1)-1;
        [temp_lval(1),temp_lat(1),temp_lon(1)] = trace_fieldline(R(1),lat_range(1),lon_range(1),time);
    end
    while isnan(temp_lat(end))
        lat_range(end) = lat_range(end)+1;
        [temp_lval(end),temp_lat(end),temp_lon(end)] = trace_fieldline(R(1),lat_range(end),lon_range(end),time);
    end
    if sum(isnan(temp_lat))==0
        j = find(lat_range < min(latf(lon(filt)==ulon(i))),1,'last');
        slat(i) = lat_range(j);
        slatc(i) = temp_lat(j);
        slonc(i) = temp_lon(j);
        slvalc(i) = temp_lval(j);
        j = find(lat_range > max(latf(lon(filt)==ulon(i))),1,'first');
        nlat(i) = lat_range(j);
        nlatc(i) = temp_lat(j);
        nlonc(i) = temp_lon(j);
        nlvalc(i) = temp_lval(j);
        continue;
    end
    j = find(isnan(temp_lat),1,'first')-1;
    slat(i) = lat_range(j);
    slatc(i) = temp_lat(j);
    slonc(i) = temp_lon(j);
    slvalc(i) = temp_lval(j);
    j = find(isnan(temp_lat),1,'last')+1;
    nlat(i) = lat_range(j);
    nlatc(i) = temp_lat(j);
    nlonc(i) = temp_lon(j);
    nlvalc(i) = temp_lval(j);
end

slat = slat(ind);
slatc = slatc(ind);
slonc = slonc(ind);
slvalc = slvalc(ind);
nlat = nlat(ind);
nlatc = nlatc(ind);
nlonc = nlonc(ind);
nlvalc = nlvalc(ind);
lat_cgm(filt) = slatc + (latf-slat)./(nlat-slat).*(nlatc-slatc);
lon_cgm(filt) = slonc + (latf-slat)./(nlat-slat).*(nlonc-slonc);
lval(filt) = slvalc + (latf-slat)./(nlat-slat).*(nlvalc-slvalc);

% for i = 1:size(lon,1)
%     filt = ~isnan(lat_cgm(i,:));
%     lat_cgm(i,:) = interp1(lat(i,filt),lat_cgm(i,filt),lat(i,:));
%     lon_cgm(i,:) = interp1(lat(i,filt),lon_cgm(i,filt),lat(i,:));
%     lval(i,:) = interp1(lat(i,filt),lval(i,filt),lat(i,:));
% end