function [bx,by,bz,br,bt,bp] = igrf(r,t,f,time)
%IGRF Calculates IGRF magnetic field components
%   [BX,BY,BZ] = IGRF(R,T,F,TIME) Calculates the cartesian magnetic field
%   components for the location(s) <R,T,F>, where R is the radial distance from
%   the center of the Earth, in units of RE, T is the geographic co-latitude in
%   radians, and F is the geographic longitude in radians. R, T, and F can be
%   scalars, vectors, or matrices, but they must be the same size. TIME gives
%   the epoch to calculate IGRF for, and must be a scalar MATLAB DATENUM
%   variable.
%
%   [...,BR,BT,BP] = IGRF(...) Calculates IGRF as above, but also provides the
%   spherical magnetic field components.
%
%   IMPORTANT NOTE:
%
%   Due to how this function calculates the IGRF magnetic field components,
%   particularly large input matrices may cause significant memory usage. To
%   prevent potential out of memory errors and/or computer freezes, this
%   function has been limited to computing 300,000 IGRF components
%   simultaenously. If a greater number of components than this are needed, they
%   should be split up into batches.
%
%   Also note that, although this function is called IGRF.m, it only uses the
%   IGRF field model for the years 1900-2015. For years outside this range,
%   other field models are used.
%
%------------------------------------------------------------------------------%
%
%   Version 1.0  Date: 2016/02/05
%
%   Copyright 2016 Aaron Hendry <aaron@hendry.co.nz>
%   Otago University Space Physics Department, Dunedin, New Zealand
%       <http://www.physics.otago.ac.nz/space>
%   Released under the Simplified BSD License:
%       <http://opensource.org/licenses/BSD-2-Clause>
%
%------------------------------------------------------------------------------%

% COMMENT OUT THESE LINES TO REMOVE MEMORY CHECKS
if numel(r) > 300000
    error('geo2cgm:igrf',['The passed matrix is too large, and may cause ',...
        'memory issues. Try again with a smaller matrix, or subdivide the ',...
        'matrix and make multiple runs']);
end

persistent gn gm curr_time g h p q acc o

if isempty(o)
    load igrf_coeff gn gm
    [g,h] = igrf_gh(time);
    curr_time = time;
    acc = 0.01;
    th = (0:acc:360)*pi/180;
    acc = (acc)*pi/180;
    [p,q] = igrf_pq(th);
    o = ones(104,1);
end
if curr_time ~= time
    [g,h] = igrf_gh(time);
    curr_time = time;
end

sz = size(t);
r = r(:)';
t = t(:)';
f = f(:)';

% Generate the P and Q legendre function coefficients.

interp = t./acc;
ind = round(t./acc);
interp = interp-ind;
interp = interp(o,:);
ind = ind+1;
P = p(:,ind).*(1-interp) + p(:,ind+1).*interp;
Q = q(:,ind).*(1-interp) + q(:,ind+1).*interp;
% [P,Q] = igrf_pq_quicker(t);

gmf = bsxfun(@times,(0:13)',f);
cf = cos(gmf);
cf = cf(gm+1,:);
sf = sin(gmf);
sf = sf(gm+1,:);
st = sin(t);
% Calculate bx, by, and bz by taking the derivative of V
rgn = cumprod(1./r(ones(15,1),:),1);
rgn = rgn(gn+2,:);
gchs = bsxfun(@times,g,cf) + bsxfun(@times,h,sf);
bt=-sum(rgn .* gchs.*Q);
bp=-sum(rgn .* (-bsxfun(@times,gm.*g,sf) + bsxfun(@times,gm.*h,cf)).*P);
bp(bp~=0)=bp(bp~=0)./st(bp~=0);
br=sum(bsxfun(@times,(gn+1),rgn) .* gchs.*P);

ct = cos(t);
sp = sin(f);
cp = cos(f);

be = br.*st + bt.*ct;

bx = reshape(be.*cp - bp.*sp,sz);
by = reshape(be.*sp + bp.*cp,sz);
bz = reshape(br.*ct - bt.*st,sz);

br = reshape(br,sz);
bt = reshape(bt,sz);
bp = reshape(bp,sz);
