function [gt,ht] = igrf_gh(time)
% IGRF_GH calculates the IGRF coefficients for the passed date
%   [GT,HT] = IGRF_GH(time) generates the g and h coefficients for IGRF
%   calculation.
%
% IGRF coefficients are calculated based on the current year (month and day can
% be used as well, if desired). The current IGRF coefficients are stored in the
% file igrf_coeff as variables G and H. G and H are both stored as [104xY]
% matrices, where Y is a variable that will change based on the current IGRF
% iteration.
%
% As of the time of writing, IGRF coefficients go back to 1900, and exist for
% every 5 years until 2015 (which means there are currently 24 sets of IGRF
% coefficients) - there are also a set of secular variation (SV) coefficients,
% which attempt to predict the changes for the next 5 years, and allow for dates
% between 2015-2020 to be used as well. This means that as of 2016, G and H are
% both 104 x 25 matrices.
%
% New sets of IGRF coefficients can easily be added to the G and H matrices, by
% appending a new column to the matrix. New coefficients are published on the
% internet in many different locations (I used the following source:
% http://www.ngdc.noaa.gov/IAGA/vmod/igrf12coeffs.txt). It is important that the
% order of the coefficients is kept the same though. Each coefficient has two
% indexes - n and m. The n-m order of the coefficients in G and H is the same,
% and follows this pattern:
%
%   n | 1  1  2  2  2  3  3  3  3 ...
%   m | 0  1  0  1  2  0  1  2  3 ...
%
% This order must be kept the same for each column, or everything will break.
% 
% Note that when adding new columns to the G and H matrices, the last column
% must ALWAYS be the SV coefficients. If for whatever reason SV coefficients
% have not been released, the last columns should be filled with zeros.
%
%------------------------------------------------------------------------------%
%
%   Version 1.0  Date: 2016/02/05
%
%   Copyright 2016 Aaron Hendry <aaron@hendry.co.nz>
%   Otago University Space Physics Department, Dunedin, New Zealand
%       <http://www.physics.otago.ac.nz/space>
%   Released under the Simplified BSD License:
%       <http://opensource.org/licenses/BSD-2-Clause>
%
%------------------------------------------------------------------------------%

persistent g h curr_time warned_lo warned_hi

if ~isempty(curr_time) && curr_time == time
    gt = g;
    ht = h;
	return
end
curr_time = time;

if isempty(warned_hi)
    warned_lo = false;
    warned_hi = false;
end

load igrf_coeff G H years

% For non-epoch years (for instance 2012), coefficients can be calculated
% by interpolating between the nearest two sets of coefficients, or using
% the SV coefficients for years greater than the greatest current epoch.

% We interpolate based on the MATLAB datenum value, as opposed to the year
% (which is used in other CGM code).
yr = datevec(time);
yr = yr(1);

% If we occur before the earliest IGRF coefficients, throw an error, because we
% have no way of extrapolating back that far.
if yr < years(1)
    error('geocgm:igrf_gh:invalid_epoch',['Requested year (%d) is',...
          ' outside the current IGRF coefficient range (1900-2015), '...
          'and is beyond the scope of the GUFM 1 model (1590-1900). At present ',...
          'extrapolating backwards in time from %d is not supported.'],...
          yr,years(1));
end

% If we're outside of the current IGRF epoch, we're using Julien Aubert's
% geodynamo-derived magnetic field model to predict the magnetic field. It's
% more accurate than using secular variation.
if yr > 2015
    if yr <= years(end)
        if ~warned_hi
            warning('geocgm:igrf_gh:epoch_warn',['Requested year (%d) is',...
                ' outside the current IGRF coefficient range (1900-2015). ',...
                ' A predictive model based on Aubert [2015] (full citation in',...
                ' comments) will be used instead. '],yr);
            warned_hi = true;
        end
    else
        if ~warned_hi
            warning('geocgm:igrf_gh:epoch_warn',['Requested year (%d) is',...
                ' outside the current IGRF coefficient range (1900-2015), and is',...
                ' beyond the scope of the Aubert [2015] model (2015-2115). Secular variation',...
                ' values will be used to extrapolate out to the required date.'],yr);
            warned_hi = true;
        end
        tyr = yr - years(end);
        interp = tyr + (time-datenum(yr,1,1))/diff(datenum([yr,yr+1],1,1));
        g = G(:,end-1)+G(:,end)*interp;
        h = H(:,end-1)+H(:,end)*interp;
        gt = g;
        ht = h;
        return;
    end
end
if yr < 1900
    if ~warned_lo
        warning('geocgm:igrf_gh:epoch_warn',['Requested year (%d) is',...
        ' outside the current IGRF coefficient range (1900-2015). ',...
        ' The GUFM 1 model will be used instead, though accuracy may be',...
        ' decreased.'],yr);
        warned_lo = true;
    end
end
df = yr-years;
df(df<0)=inf;
[~,ind] = min(df);
tyr = years(ind);

f2 = (time-datenum(tyr,1,1))./diff(datenum([tyr,years(ind+1)],1,1));
f1 = 1-f2;
g = G(:,ind)*f1 + G(:,ind+1)*f2;
h = H(:,ind)*f1 + H(:,ind+1)*f2;
gt = g;
ht = h;




