function [x,y,z] = step(x,y,z,ds,time)
% STEP steps along an IGRF magnetic field line by distance ds
%   [X,Y,Z] = STEP(X,Y,Z,DS,TIME) Calculates the cartesian location of the IGRF
%   field line that passes through the point <X,Y,Z>, a distance ds along the
%   field line from the original point. The IGRF coefficients for the epoch TIME
%   are used, where TIME is a scalar MATLAB DATENUM variable. X, Y, Z, and DS
%   can be scalars, vectors, or matrices, but must all be the same size.
%
%   This algorithm calculates the next step along an IGRF magnetic field line
%   using the Runge-Kutta-Merson method.
%
%------------------------------------------------------------------------------%
%
%   Version 1.0  Date: 2016/02/05
%
%   Copyright 2016 Aaron Hendry <aaron@hendry.co.nz>
%   Otago University Space Physics Department, Dunedin, New Zealand
%       <http://www.physics.otago.ac.nz/space>
%   Released under the Simplified BSD License:
%       <http://opensource.org/licenses/BSD-2-Clause>
%
%------------------------------------------------------------------------------%

ds = -ds/3;
[r11,r12,r13] = step_fun(x,y,z,ds,time);
[r21,r22,r23] = step_fun(x+r11,y+r12,z+r13,ds,time);
[r31,r32,r33] = step_fun(x+0.5*(r11+r21),y+0.5*(r12+r22),z+0.5*(r13+r23),ds,time);
[r41,r42,r43] = step_fun(x+0.375.*(r11+3*r31),y+0.375.*(r12+3*r32),...
                      z+0.375.*(r13+3*r33),ds,time);
[r51,r52,r53] = step_fun(x+1.5*(r11-3*r31+4*r41),y+1.5*(r12-3*r32+4*r42),...
                      z+1.5*(r13-3*r33+4*r43),ds,time);

x = x+0.5*(r11+4*r41+r51);
y = y+0.5*(r12+4*r42+r52);
z = z+0.5*(r13+4*r43+r53);

end

function [r1,r2,r3] = step_fun(x,y,z,ds,time)
% Aux function for the step function

[x,y,z] = cart2sph(x,y,z);
[bx,by,bz] = igrf(x,y,z,time);

B = ds./sqrt(bx.^2+by.^2+bz.^2);
r1 = bx.*B;
r2 = by.*B;
r3 = bz.*B;

end
